@@ -1,3 +1,8 @@
+'''
+    Best Effort Callback.
+    For instructions on how to test in a simulated environment please see:
+        tests/interactive/best_effort_cb.py
+'''
 from cycler import cycler
 from datetime import datetime
 from io import StringIO
@@ -17,15 +22,17 @@ from .fitting import PeakStats
 
 
 class BestEffortCallback(CallbackBase):
-    def __init__(self):
+    def __init__(self, *, fig_factory=None, table_enabled=True):
         # internal state
         self._start_doc = None
         self._descriptors = {}
         self._table = None
         self._heading_enabled = True
-        self._table_enabled = True
+        self._table_enabled = table_enabled
         self._baseline_enabled = True
         self._plots_enabled = True
+        # axes supplied from outside
+        self._fig_factory = fig_factory
         # maps descriptor uid to dict which maps data key to LivePlot instance
         self._live_plots = {}
         self._live_grids = {}
@@ -111,9 +118,20 @@ class BestEffortCallback(CallbackBase):
             dimensions = GUESS  # Fall back on our GUESS.
             warn("We are ignoring the dimensions hinted because we cannot "
                  "combine streams.")
-        # TODO : Add option to select multiple dim_fields
+
+        # for each dimension, choose one field only
+        # the plan can supply a list of fields. It's assumed the first
+        # of the list is always the one plotted against
         self.dim_fields = [fields[0]
                            for fields, stream_name in dimensions]
+
+        # make distinction between flattened fields and plotted fields
+        # motivation for this is that when plotting, we find dependent variable
+        # by finding elements that are not independent variables
+        self.all_dim_fields = [field
+                               for fields, stream_name in dimensions
+                               for field in fields]
+
         _, self.dim_stream = dimensions[0]
 
         # Print heading.
@@ -144,6 +162,11 @@ class BestEffortCallback(CallbackBase):
             self._cleanup_motor_heuristic = False
             fixed_dim_fields = []
             for obj_name in self.dim_fields:
+                # Special case: 'time' can be a dim_field, but it's not an
+                # object name. Just add it directly to the list of fields.
+                if obj_name == 'time':
+                    fixed_dim_fields.append('time')
+                    continue
                 try:
                     fields = doc.get('hints', {}).get(obj_name, {})['fields']
                 except KeyError:
@@ -156,10 +179,11 @@ class BestEffortCallback(CallbackBase):
         if stream_name == self.dim_stream:
             # Ensure that no independent variables ('dimensions') are
             # duplicated here.
-            columns = [c for c in columns if c not in self.dim_fields]
+            columns = [c for c in columns if c not in self.all_dim_fields]
 
             if self._table_enabled:
-                self._table = LiveTable(list(self.dim_fields) + columns)
+                # plot everything, independent or dependent variables
+                self._table = LiveTable(list(self.all_dim_fields) + columns)
                 self._table('start', self._start_doc)
                 self._table('descriptor', doc)
 
@@ -209,7 +233,10 @@ class BestEffortCallback(CallbackBase):
             # we need 1 or 2 dims to do anything, do not make empty figures
             return
 
-        fig = plt.figure(fig_name)
+        if self._fig_factory:
+            fig = self._fig_factory(fig_name)
+        else:
+            fig = plt.figure(fig_name)
         if not fig.axes:
             # This is apparently a fresh figure. Make axes.
             # The complexity here is due to making a shared x axis. This can be
@@ -263,7 +290,7 @@ class BestEffortCallback(CallbackBase):
             gridding = self._start_doc.get('hints', {}).get('gridding')
             if gridding == 'rectilinear':
                 self._live_grids[doc['uid']] = {}
-                x_key, y_key = dim_fields
+                slow, fast = dim_fields
                 try:
                     extents = self._start_doc['extents']
                     shape = self._start_doc['shape']
@@ -289,7 +316,7 @@ class BestEffortCallback(CallbackBase):
                             ax.set_aspect(aspect, adjustable='datalim')
 
                         live_grid = LiveGrid(shape, I_key,
-                                             xlabel=x_key, ylabel=y_key,
+                                             xlabel=fast, ylabel=slow,
                                              extent=adjusted_extent,
                                              aspect=aspect,
                                              ax=ax)
