@@ -1,8 +1,9 @@
 import sys
-import itertools
+import inspect
 from itertools import chain
+from functools import partial
 
-from collections import (Iterable, defaultdict)
+from collections import defaultdict
 import time
 
 import numpy as np
@@ -57,59 +58,13 @@ def count(detectors, num=1, delay=None, *, md=None):
     _md.update(md or {})
     _md['hints'].setdefault('dimensions', [(('time',), 'primary')])
 
-    # If delay is a scalar, repeat it forever. If it is an iterable, leave it.
-    if not isinstance(delay, Iterable):
-        delay = itertools.repeat(delay)
-    else:
-        try:
-            num_delays = len(delay)
-        except TypeError:
-            # No way to tell in advance if we have enough delays.
-            pass
-        else:
-            if num - 1 > num_delays:
-                raise ValueError("num=%r but delays only provides %r "
-                                 "entries" % (num, num_delays))
-        delay = iter(delay)
-
     @bpp.stage_decorator(detectors)
     @bpp.run_decorator(md=_md)
-    def finite_plan():
-        for i in range(num):
-            now = time.time()  # Intercept the flow in its earliest moment.
-            yield Msg('checkpoint')
-            yield from bps.trigger_and_read(detectors)
-            try:
-                d = next(delay)
-            except StopIteration:
-                if i + 1 == num:
-                    break
-                else:
-                    # num specifies a number of iterations less than delay
-                    raise ValueError("num=%r but delays only provides %r "
-                                     "entries" % (num, i))
-            if d is not None:
-                d = d - (time.time() - now)
-                if d > 0:  # Sleep if and only if time is left to do it.
-                    yield Msg('sleep', None, d)
+    def inner_count():
+        return (yield from bps.repeat(partial(bps.trigger_and_read, detectors),
+                                      num=num, delay=delay))
 
-    @bpp.stage_decorator(detectors)
-    @bpp.run_decorator(md=_md)
-    def infinite_plan():
-        while True:
-            yield Msg('checkpoint')
-            yield from bps.trigger_and_read(detectors)
-            try:
-                d = next(delay)
-            except StopIteration:
-                break
-            if d is not None:
-                yield Msg('sleep', None, d)
-
-    if num is None:
-        return (yield from infinite_plan())
-    else:
-        return (yield from finite_plan())
+    return (yield from inner_count())
 
 
 def list_scan(detectors, motor, steps, *, per_step=None, md=None):
@@ -121,11 +76,11 @@ def list_scan(detectors, motor, steps, *, per_step=None, md=None):
     detectors : list
         list of 'readable' objects
     motor : object
-        any 'setable' object (motor, temp controller, etc.)
+        any 'settable' object (motor, temp controller, etc.)
     steps : list
         list of positions
     per_step : callable, optional
-        hook for cutomizing action of inner loop (messages per step)
+        hook for customizing action of inner loop (messages per step)
         Expected signature:
         ``f(detectors, motor, step) -> plan (a generator)``
     md : dict, optional
@@ -176,11 +131,11 @@ def rel_list_scan(detectors, motor, steps, *, per_step=None, md=None):
     detectors : list
         list of 'readable' objects
     motor : object
-        any 'setable' object (motor, temp controller, etc.)
+        any 'settable' object (motor, temp controller, etc.)
     steps : list
         list of positions relative to current position
     per_step : callable, optional
-        hook for cutomizing action of inner loop (messages per step)
+        hook for customizing action of inner loop (messages per step)
         Expected signature: ``f(detectors, motor, step)``
     md : dict, optional
         metadata
@@ -201,7 +156,7 @@ def rel_list_scan(detectors, motor, steps, *, per_step=None, md=None):
     return (yield from inner_relative_list_scan())
 
 
-def scan(detectors, motor, start, stop, num, *, per_step=None, md=None):
+def _scan_1d(detectors, motor, start, stop, num, *, per_step=None, md=None):
     """
     Scan over one variable in equally spaced steps.
 
@@ -210,7 +165,7 @@ def scan(detectors, motor, start, stop, num, *, per_step=None, md=None):
     detectors : list
         list of 'readable' objects
     motor : object
-        any 'setable' object (motor, temp controller, etc.)
+        any 'settable' object (motor, temp controller, etc.)
     start : float
         starting position of motor
     stop : float
@@ -218,7 +173,7 @@ def scan(detectors, motor, start, stop, num, *, per_step=None, md=None):
     num : int
         number of steps
     per_step : callable, optional
-        hook for cutomizing action of inner loop (messages per step)
+        hook for customizing action of inner loop (messages per step)
         Expected signature: ``f(detectors, motor, step)``
     md : dict, optional
         metadata
@@ -263,8 +218,8 @@ def scan(detectors, motor, start, stop, num, *, per_step=None, md=None):
     return (yield from inner_scan())
 
 
-def rel_scan(detectors, motor, start, stop, num, *, per_step=None,
-             md=None):
+def _rel_scan_1d(detectors, motor, start, stop, num, *, per_step=None,
+                 md=None):
     """
     Scan over one variable in equally spaced steps relative to current positon.
 
@@ -273,7 +228,7 @@ def rel_scan(detectors, motor, start, stop, num, *, per_step=None,
     detectors : list
         list of 'readable' objects
     motor : object
-        any 'setable' object (motor, temp controller, etc.)
+        any 'settable' object (motor, temp controller, etc.)
     start : float
         starting position of motor
     stop : float
@@ -281,7 +236,7 @@ def rel_scan(detectors, motor, start, stop, num, *, per_step=None,
     num : int
         number of steps
     per_step : callable, optional
-        hook for cutomizing action of inner loop (messages per step)
+        hook for customizing action of inner loop (messages per step)
         Expected signature: ``f(detectors, motor, step)``
     md : dict, optional
         metadata
@@ -297,8 +252,8 @@ def rel_scan(detectors, motor, start, stop, num, *, per_step=None,
     @bpp.reset_positions_decorator([motor])
     @bpp.relative_set_decorator([motor])
     def inner_relative_scan():
-        return (yield from scan(detectors, motor, start, stop,
-                                num, per_step=per_step, md=_md))
+        return (yield from _scan_1d(detectors, motor, start, stop,
+                                    num, per_step=per_step, md=_md))
 
     return (yield from inner_relative_scan())
 
@@ -312,7 +267,7 @@ def log_scan(detectors, motor, start, stop, num, *, per_step=None, md=None):
     detectors : list
         list of 'readable' objects
     motor : object
-        any 'setable' object (motor, temp controller, etc.)
+        any 'settable' object (motor, temp controller, etc.)
     start : float
         starting position of motor
     stop : float
@@ -320,7 +275,7 @@ def log_scan(detectors, motor, start, stop, num, *, per_step=None, md=None):
     num : int
         number of steps
     per_step : callable, optional
-        hook for cutomizing action of inner loop (messages per step)
+        hook for customizing action of inner loop (messages per step)
         Expected signature: ``f(detectors, motor, step)``
     md : dict, optional
         metadata
@@ -375,7 +330,7 @@ def rel_log_scan(detectors, motor, start, stop, num, *, per_step=None,
     detectors : list
         list of 'readable' objects
     motor : object
-        any 'setable' object (motor, temp controller, etc.)
+        any 'settable' object (motor, temp controller, etc.)
     start : float
         starting position of motor
     stop : float
@@ -383,7 +338,7 @@ def rel_log_scan(detectors, motor, start, stop, num, *, per_step=None,
     num : int
         number of steps
     per_step : callable, optional
-        hook for cutomizing action of inner loop (messages per step)
+        hook for customizing action of inner loop (messages per step)
         Expected signature: ``f(detectors, motor, step)``
     md : dict, optional
         metadata
@@ -418,7 +373,7 @@ def adaptive_scan(detectors, target_field, motor, start, stop,
     target_field : string
         data field whose output is the focus of the adaptive tuning
     motor : object
-        any 'setable' object (motor, temp controller, etc.)
+        any 'settable' object (motor, temp controller, etc.)
     start : float
         starting position of motor
     stop : float
@@ -530,7 +485,7 @@ def rel_adaptive_scan(detectors, target_field, motor, start, stop,
     target_field : string
         data field whose output is the focus of the adaptive tuning
     motor : object
-        any 'setable' object (motor, temp controller, etc.)
+        any 'settable' object (motor, temp controller, etc.)
     start : float
         starting position of motor
     stop : float
@@ -603,7 +558,7 @@ def tune_centroid(
     signal : string
         detector field whose output is to maximize
     motor : object
-        any 'setable' object (motor, temp controller, etc.)
+        any 'settable' object (motor, temp controller, etc.)
     start : float
         start of range
     stop : float
@@ -614,7 +569,7 @@ def tune_centroid(
         number of points with each traversal, default = 10
     step_factor : float, optional
         used in calculating new range after each pass
-        
+
         note: step_factor > 1.0, default = 3
     snake : bool, optional
         if False (default), always scan from start to stop
@@ -720,7 +675,7 @@ def scan_nd(detectors, cycler, *, per_step=None, md=None):
     cycler : Cycler
         list of dictionaries mapping motors to positions
     per_step : callable, optional
-        hook for cutomizing action of inner loop (messages per step)
+        hook for customizing action of inner loop (messages per step)
         See docstring of bluesky.plan_stubs.one_nd_step (the default) for
         details.
     md : dict, optional
@@ -758,6 +713,30 @@ def scan_nd(detectors, cycler, *, per_step=None, md=None):
 
     if per_step is None:
         per_step = bps.one_nd_step
+    else:
+        # Ensure that the user-defined per-step has the expected signature.
+        sig = inspect.signature(per_step)
+        if sig == inspect.signature(bps.one_nd_step):
+            pass
+        elif sig == inspect.signature(bps.one_1d_step):
+            # Accept this signature for back-compat reasons (because
+            # inner_product_scan was renamed scan).
+            dims = len(list(cycler.keys))
+            if dims != 1:
+                raise TypeError("Signature of per_step assumes 1D trajectory "
+                                "but {} motors are specified.".format(dims))
+            motor, = cycler.keys
+            user_per_step = per_step
+            def adapter(detectors, step, pos_cache):
+                # one_nd_step 'step' parameter is a dict; one_id_step 'step'
+                # parameter is a value
+                step, = step.values()
+                return (yield from user_per_step(detectors, motor, step))
+            per_step = adapter
+        else:
+            raise TypeError("per_step must be a callable with the signature "
+                            "<Signature (detectors, step, pos_cache)> or "
+                            "<Signature (detectors, motor, step)>.")
     pos_cache = defaultdict(lambda: None)  # where last position is stashed
     cycler = utils.merge_cycler(cycler)
     motors = list(cycler.keys)
@@ -772,6 +751,14 @@ def scan_nd(detectors, cycler, *, per_step=None, md=None):
 
 
 def inner_product_scan(detectors, num, *args, per_step=None, md=None):
+    # For scan, num is the _last_ positional arg instead of the first one.
+    # Notice the swapped order here.
+    md = md or {}
+    md.setdefault('plan_name', 'inner_product_scan')
+    yield from scan(detectors, *args, num, per_step=None, md=md)
+
+
+def scan(detectors, *args, num=None, per_step=None, md=None):
     """
     Scan over one multi-motor trajectory.
 
@@ -779,13 +766,22 @@ def inner_product_scan(detectors, num, *args, per_step=None, md=None):
     ----------
     detectors : list
         list of 'readable' objects
+    *args :
+        For one dimension, ``motor, start, stop``.
+        In general:
+
+        .. code-block:: python
+
+            motor1, start1, stop1,
+            motor2, start2, start2,
+            ...,
+            motorN, startN, stopN
+
+        Motors can be any 'settable' object (motor, temp controller, etc.)
     num : integer
-        number of steps
-    ``*args`` : {Positioner, Positioner, int}
-        patterned like (``motor1, start1, stop1, ..., motorN, startN, stopN``)
-        Motors can be any 'setable' object (motor, temp controller, etc.)
+        number of points
     per_step : callable, optional
-        hook for cutomizing action of inner loop (messages per step)
+        hook for customizing action of inner loop (messages per step)
         See docstring of bluesky.plan_stubs.one_nd_step (the default) for
         details.
     md : dict, optional
@@ -797,20 +793,55 @@ def inner_product_scan(detectors, num, *args, per_step=None, md=None):
     :func:`bluesky.plans.grid_scan`
     :func:`bluesky.plans.scan_nd`
     """
+    # For back-compat reasons, we accept 'num' as the last positional argument:
+    # scan(detectors, motor, -1, 1, 3)
+    # or by keyword:
+    # scan(detectors, motor, -1, 1, num=3)
+    # ... which requires some special processing.
+    if num is None:
+        if len(args) % 3 != 1:
+            raise ValueError("The number of points to scan must be provided "
+                             "as the last positional argument or as keyword "
+                             "argument 'num'.")
+        num = args[-1]
+        args = args[:-1]
     md_args = list(chain(*((repr(motor), start, stop)
                            for motor, start, stop in partition(3, args))))
     motor_names = tuple(motor.name for motor, start, stop
                         in partition(3, args))
+    md = md or {}
     _md = {'plan_args': {'detectors': list(map(repr, detectors)),
                          'num': num, 'args': md_args,
                          'per_step': repr(per_step)},
-           'plan_name': 'inner_product_scan',
+           'plan_name': 'scan',
            'plan_pattern': 'inner_product',
            'plan_pattern_module': plan_patterns.__name__,
            'plan_pattern_args': dict(num=num, args=md_args),
            'motors': motor_names
            }
-    _md.update(md or {})
+    _md.update(md)
+
+    # get hints for best effort callback
+    motors = [motor for motor, start, stop in partition(3, args)]
+
+    # Give a hint that the motors all lie along the same axis
+    # [(['motor1', 'motor2', ...], 'primary'), ] is 1D (this case)
+    # [ ('motor1', 'primary'), ('motor2', 'primary'), ... ] is 2D for example
+    # call x_fields because these are meant to be the x (independent) axis
+    x_fields = []
+    for motor in motors:
+        x_fields.extend(getattr(motor, 'hints', {}).get('fields', []))
+
+    default_dimensions = [(x_fields, 'primary')]
+
+    default_hints = {}
+    if len(x_fields) > 0:
+        default_hints.update(dimensions=default_dimensions)
+
+    # now add default_hints and override any hints from the original md (if
+    # exists)
+    _md['hints'] = default_hints
+    _md['hints'].update(md.get('hints', {}) or {})
 
     full_cycler = plan_patterns.inner_product(num=num, args=args)
 
@@ -827,7 +858,7 @@ def grid_scan(detectors, *args, per_step=None, md=None):
     detectors : list
         list of 'readable' objects
     ``*args``
-        patterned like (``motor1, start1, stop1, num1,```
+        patterned like (``motor1, start1, stop1, num1,``
                         ``motor2, start2, stop2, num2, snake2,``
                         ``motor3, start3, stop3, num3, snake3,`` ...
                         ``motorN, startN, stopN, numN, snakeN``)
@@ -837,7 +868,7 @@ def grid_scan(detectors, *args, per_step=None, md=None):
         indicating whether to following snake-like, winding trajectory or a
         simple left-to-right trajectory.
     per_step : callable, optional
-        hook for cutomizing action of inner loop (messages per step)
+        hook for customizing action of inner loop (messages per step)
         See docstring of bluesky.plan_stubs.one_nd_step (the default) for
         details.
     md : dict, optional
@@ -903,13 +934,13 @@ def rel_grid_scan(detectors, *args, per_step=None, md=None):
     ``*args``
         patterned like ``motor1, start1, stop1, num1, motor2, start2, stop2,
         num2, snake2,`` ..., ``motorN, startN, stopN, numN, snakeN``
-        Motors can be any 'setable' object (motor, temp controller, etc.)
+        Motors can be any 'settable' object (motor, temp controller, etc.)
         Notice that the first motor is followed by start, stop, num.
         All other motors are followed by start, stop, num, snake where snake
         is a boolean indicating whether to following snake-like, winding
         trajectory or a simple left-to-right trajectory.
     per_step : callable, optional
-        hook for cutomizing action of inner loop (messages per step)
+        hook for customizing action of inner loop (messages per step)
         See docstring of bluesky.plan_stubs.one_nd_step (the default) for
         details.
     md : dict, optional
@@ -936,6 +967,14 @@ def rel_grid_scan(detectors, *args, per_step=None, md=None):
 
 
 def relative_inner_product_scan(detectors, num, *args, per_step=None, md=None):
+    # For rel_scan, num is the _last_ positional arg instead of the first one.
+    # Notice the swapped order here.
+    md = md or {}
+    md.setdefault('plan_name', 'relative_inner_product_scan')
+    yield from rel_scan(detectors, *args, num, per_step=per_step, md=md)
+
+
+def rel_scan(detectors, *args, num=None, per_step=None, md=None):
     """
     Scan over one multi-motor trajectory relative to current position.
 
@@ -943,13 +982,22 @@ def relative_inner_product_scan(detectors, num, *args, per_step=None, md=None):
     ----------
     detectors : list
         list of 'readable' objects
+    *args :
+        For one dimension, ``motor, start, stop``.
+        In general:
+
+        .. code-block:: python
+
+            motor1, start1, stop1,
+            motor2, start2, start2,
+            ...,
+            motorN, startN, stopN,
+
+        Motors can be any 'settable' object (motor, temp controller, etc.)
     num : integer
-        number of steps
-    ``*args``
-        patterned like (``motor1, start1, stop1, ..., motorN, startN, stopN``)
-        Motors can be any 'setable' object (motor, temp controller, etc.)
+        number of points
     per_step : callable, optional
-        hook for cutomizing action of inner loop (messages per step)
+        hook for customizing action of inner loop (messages per step)
         See docstring of bluesky.plan_stubs.one_nd_step (the default) for
         details.
     md : dict, optional
@@ -961,37 +1009,18 @@ def relative_inner_product_scan(detectors, num, *args, per_step=None, md=None):
     :func:`bluesky.plans.inner_product_scan`
     :func:`bluesky.plans.scan_nd`
     """
-    _md = {'plan_name': 'relative_inner_product_scan',
-           }
+    _md = {'plan_name': 'rel_scan'}
     md = md or {}
     _md.update(md)
     motors = [motor for motor, start, stop in partition(3, args)]
 
-    # Give a hint that the motors all lie along the same axis
-    # [(['motor1', 'motor2', ...], 'primary'), ] is 1D (this case)
-    # [ ('motor1', 'primary'), ('motor2', 'primary'), ... ] is 2D for example
-    fields = []
-    for motor in motors:
-        fields.extend(getattr(motor, 'hints', {}).get('fields', []))
-
-    default_dimensions = [(fields, 'primary')]
-
-    default_hints = {}
-    if len(fields) > 0:
-        default_hints.update(dimensions=default_dimensions)
-
-    # now add default_hints and override any hints from the original md (if
-    # exists)
-    _md['hints'] = default_hints
-    _md['hints'].update(md.get('hints', {}) or {})
-
     @bpp.reset_positions_decorator(motors)
     @bpp.relative_set_decorator(motors)
-    def inner_relative_inner_product_scan():
-        return (yield from inner_product_scan(detectors, num, *args,
-                                              per_step=per_step, md=_md))
+    def inner_rel_scan():
+        return (yield from scan(detectors, *args, num=num,
+                                per_step=per_step, md=_md))
 
-    return (yield from inner_relative_inner_product_scan())
+    return (yield from inner_rel_scan())
 
 
 def tweak(detector, target_field, motor, step, *, md=None):
@@ -1080,9 +1109,9 @@ def spiral_fermat(detectors, x_motor, y_motor, x_start, y_start, x_range,
     detectors : list
         list of 'readable' objects
     x_motor : object
-        any 'setable' object (motor, temp controller, etc.)
+        any 'settable' object (motor, temp controller, etc.)
     y_motor : object
-        any 'setable' object (motor, temp controller, etc.)
+        any 'settable' object (motor, temp controller, etc.)
     x_start : float
         x center
     y_start : float
@@ -1098,7 +1127,7 @@ def spiral_fermat(detectors, x_motor, y_motor, x_start, y_start, x_range,
     tilt : float, optional
         Tilt angle in radians, default 0.0
     per_step : callable, optional
-        hook for cutomizing action of inner loop (messages per step)
+        hook for customizing action of inner loop (messages per step)
         See docstring of bluesky.plan_stubs.one_nd_step (the default) for
         details.
     md : dict, optional
@@ -1153,9 +1182,9 @@ def rel_spiral_fermat(detectors, x_motor, y_motor, x_range, y_range, dr,
     detectors : list
         list of 'readable' objects
     x_motor : object
-        any 'setable' object (motor, temp controller, etc.)
+        any 'settable' object (motor, temp controller, etc.)
     y_motor : object
-        any 'setable' object (motor, temp controller, etc.)
+        any 'settable' object (motor, temp controller, etc.)
     x_range : float
         x width of spiral
     y_range : float
@@ -1167,7 +1196,7 @@ def rel_spiral_fermat(detectors, x_motor, y_motor, x_range, y_range, dr,
     tilt : float, optional
         Tilt angle in radians, default 0.0
     per_step : callable, optional
-        hook for cutomizing action of inner loop (messages per step)
+        hook for customizing action of inner loop (messages per step)
         See docstring of bluesky.plan_stubs.one_nd_step (the default) for
         details.
     md : dict, optional
@@ -1195,9 +1224,9 @@ def spiral(detectors, x_motor, y_motor, x_start, y_start, x_range, y_range, dr,
     Parameters
     ----------
     x_motor : object
-        any 'setable' object (motor, temp controller, etc.)
+        any 'settable' object (motor, temp controller, etc.)
     y_motor : object
-        any 'setable' object (motor, temp controller, etc.)
+        any 'settable' object (motor, temp controller, etc.)
     x_start : float
         x center
     y_start : float
@@ -1213,7 +1242,7 @@ def spiral(detectors, x_motor, y_motor, x_start, y_start, x_range, y_range, dr,
     tilt : float, optional
         Tilt angle in radians, default 0.0
     per_step : callable, optional
-        hook for cutomizing action of inner loop (messages per step)
+        hook for customizing action of inner loop (messages per step)
         See docstring of bluesky.plan_stubs.one_nd_step (the default) for
         details.
     md : dict, optional
@@ -1266,9 +1295,9 @@ def rel_spiral(detectors, x_motor, y_motor, x_range, y_range, dr, nth,
     Parameters
     ----------
     x_motor : object
-        any 'setable' object (motor, temp controller, etc.)
+        any 'settable' object (motor, temp controller, etc.)
     y_motor : object
-        any 'setable' object (motor, temp controller, etc.)
+        any 'settable' object (motor, temp controller, etc.)
     x_start : float
         x center
     y_start : float
@@ -1284,7 +1313,7 @@ def rel_spiral(detectors, x_motor, y_motor, x_range, y_range, dr, nth,
     tilt : float, optional
         Tilt angle in radians, default 0.0
     per_step : callable, optional
-        hook for cutomizing action of inner loop (messages per step)
+        hook for customizing action of inner loop (messages per step)
         See docstring of bluesky.plan_stubs.one_nd_step (the default) for
         details.
     md : dict, optional
@@ -1341,7 +1370,7 @@ def ramp_plan(go_plan,
 
         In seconds
 
-    period : fload, optional
+    period : float, optional
         If not None, take data no faster than this.  If None, take
         data as fast as possible
 
